﻿using C1.DataCollection;
using C1.DataFilter;
using C1.WPF.DataFilter;
using FilterEditorDemo.Models;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FilterEditorDemo.CustomControls
{
    /// <summary>
    /// Interaction logic for RuleTree.xaml
    /// </summary>
    public partial class RuleTree : UserControl
    {
        ObservableCollection<Rule> Rules { get; set; }
        RuleEditorControl ruleEditor;
        Rule parentForNewRule;

        #region Object Model
        public IEnumerable Source { get; set; }
        public C1.DataFilter.Expression Expression
        {
            get
            {
                return (ruleTree.SelectedItem?.DataContext as Rule)?.CombinedExpression;
            }
        }
        public EventHandler RuleChanged;
        #endregion
        public RuleTree()
        {
            InitializeComponent();
            InitRules();
        }
        private void InitRules()
        {
            var all = new Rule { Name = "All", CanDelete = false, IsExpanded = true};
            var inbox = new Rule
            {
                Name = "Inbox",
                ParentRule = all,
                IsSelected = true,
                CanDelete = false,
                Expression = new OperationExpression() { FilterOperation= FilterOperation.Equal, PropertyName = "SendByMe", Value = false}
            };
            var sentItems = new Rule
            {
                Name = "Sent Items",
                ParentRule = all,
                CanDelete = false,
                Expression = new OperationExpression() { FilterOperation = FilterOperation.Equal, PropertyName = "SendByMe", Value = true }
            };
            all.ChildRules.Add(inbox);
            all.ChildRules.Add(sentItems);

            Rules = new ObservableCollection<Rule>() { all };
            ruleTree.ItemsSource = Rules;
        }

        private void OpenRuleEditor(Rule rule, FrameworkElement element)
        {
            ruleEditor = new RuleEditorControl(rule, Source);
            popupCtrl.Child = ruleEditor;
            popup.PlacementTarget = element;
            ruleEditor.RuleChanged += RuleEditor_RuleChanged;
            popup.IsOpen = true;
        }
        private void BtnAddRule_Click(object sender, RoutedEventArgs e)
        {
            var rule = (sender as Button).DataContext as Rule;
            var newRule = new Rule() { Name = "New Rule" };
            parentForNewRule = rule;
            OpenRuleEditor(newRule, sender as Button);
        }

        private void BtnRuleEditor_Click(object sender, RoutedEventArgs e)
        {
            var rule = (sender as Button).DataContext as Rule;
            OpenRuleEditor(rule, sender as Button);
        }

        private void RuleEditor_RuleChanged(object sender, EventArgs e)
        {
            ruleEditor.RuleChanged -= RuleEditor_RuleChanged;

            // If Newly added rule
            if (ruleEditor.Rule.ParentRule == null && ruleEditor.Rule.ChildRules.Count == 0)
            {
                ruleEditor.Rule.ParentRule = parentForNewRule;
                parentForNewRule.ChildRules.Add(ruleEditor.Rule);
                parentForNewRule.IsExpanded = true;
                ruleEditor.Rule.IsSelected = true;
            }

            popup.IsOpen = false;
        }

        private void DeleteRule_Click(object sender, RoutedEventArgs e)
        {
            var rule = (sender as Button).DataContext as Rule;
            rule.ParentRule.ChildRules.Remove(rule);
        }
        private void RuleTree_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (ruleTree.SelectedItem!=null)
            {
                RuleChanged?.Invoke(this, EventArgs.Empty);
            }
        }

        private void PopupCtrl_ClosePopup(object sender, EventArgs e)
        {
            popup.IsOpen = false;
        }
    }
}
