﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using C1.WPF.Theming;
using C1.WPF.Theming.Material;
using C1.WPF.Theming.MaterialDark;
using C1.WPF.Theming.Office2016;
using C1.WPF.Theming.ExpressionDark;
using C1.WPF.Theming.ExpressionLight;
using C1.WPF.Theming.WhistlerBlue;
using C1.WPF.Theming.ShinyBlue;

namespace FilterEditorDemo.Service
{
    public class ThemeService
    {
        public static void ApplyTheme(AvailableThemes themeToApply, FrameworkElement element)
        {
            var theme = GetTheme(themeToApply);
            theme.Apply(element);

            // Common Color Maps
            MapColorResource("lightColor", "SelectedBackground", theme);
            MapColorResource("darkColor", "SelectedBackground", theme);
            MapColorResource("foreground", "TextBrush", theme);
            MapColorResource("background", "InputBackgroundBrush", theme);
        }
        public static C1Theme GetTheme(AvailableThemes themeName)
        {
            C1Theme theme = null;
            switch (themeName)
            {
                case AvailableThemes.OfficeWhite:
                    theme = new C1ThemeOffice2016White();
                    break;
                case AvailableThemes.OfficeColorful:
                    theme = new C1ThemeOffice2016Colorful();
                    break;
                case AvailableThemes.OfficeBlack:
                    theme = new C1ThemeOffice2016Black();
                    break;
                case AvailableThemes.Material:
                    theme = new C1ThemeMaterial();
                    break;
                case AvailableThemes.MaterialDark:
                    theme = new C1ThemeMaterialDark();
                    break;
                case AvailableThemes.ShinyBlue:
                    theme = new C1ThemeShinyBlue();
                    break;
                case AvailableThemes.WhistlerBlue:
                    theme = new C1ThemeWhistlerBlue();
                    break;
                case AvailableThemes.ExpressionLight:
                    theme = new C1ThemeExpressionLight();
                    break;
                case AvailableThemes.ExpressionDark:
                    theme = new C1ThemeExpressionDark();
                    break;
            }
            return theme;
        }
        private static void MapColorResource(string fromRes, string toRes, C1Theme theme)
        {
            var from = (App.Current.Resources[fromRes] as SolidColorBrush);
            from = from.Clone();
            var to = (theme.TryFindResource(toRes) as SolidColorBrush);
            if (to != null)
                from.Color = to.Color;
            App.Current.Resources[fromRes] = from;
        }
        public enum AvailableThemes
        {
            OfficeWhite,
            OfficeColorful,
            OfficeBlack,
            Material,
            MaterialDark,
            ShinyBlue,
            WhistlerBlue,
            ExpressionLight,
            ExpressionDark
        }
    }
}
