﻿using FilterEditorDemo.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Mail;
using C1.DataCollection;
using Google.Apis.Auth.OAuth2;
using Google.Apis.Auth.OAuth2.Flows;
using Google.Apis.Util.Store;
using System.Threading;
using Google.Apis.Util;
using MailKit.Security;
using MailKit.Net.Imap;
using MailKit.Search;
using MimeKit;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using MailKit;
using System.ComponentModel;
using System.Windows;

namespace FilterEditorDemo.Service
{
    public class UserService: INotifyPropertyChanged
    {
        static UserService instance;
        ImapClient client;
        IMailFolder folder;
        CancellationTokenSource cancel;
        int _totalCount;
        SemaphoreSlim _stateAsyncLock;
        private UserService(){}

        #region Object Model
        public string User { get; set; }
        public C1DataCollection<Message> Messages { get; set; }
        public int TotalCount
        {
            get { return _totalCount; }
            private set { _totalCount = value; OnPropertyChanged(nameof(TotalCount)); }
        }

        public event EventHandler<int> FetchedMessages;
        public event PropertyChangedEventHandler PropertyChanged;
        public event EventHandler<string> FetchedMessageContent;
        #endregion
        public static UserService GetInstance()
        {
            if (instance == null)
            {
                instance = new UserService();
                instance._stateAsyncLock = new SemaphoreSlim(1);
            }
            return instance;
        }

        public async Task Login(string userName, string password)
        {
            this.User = userName;
            this.client = new ImapClient();
            cancel = new CancellationTokenSource();
            await client.ConnectAsync("imap.gmail.com", 993, SecureSocketOptions.SslOnConnect, cancel.Token);
            await client.AuthenticateAsync(userName, password, cancel.Token);

            FetchMessages();
        }

        public async Task LoginWithGmail(string userName)
        {
            this.User = userName;

            var clientSecrets = new ClientSecrets
            {
                ClientId = "11616510429-r9j1iq5rm09odka27nb9752b5ab669rs.apps.googleusercontent.com",
                ClientSecret = "qJ-Zyl4GebEflrJDhPCX2jLW"
            };

            var codeFlow = new GoogleAuthorizationCodeFlow(new GoogleAuthorizationCodeFlow.Initializer
            {
                DataStore = new FileDataStore("CredentialCacheFolder", false),
                Scopes = new[] { "https://mail.google.com/" },
                ClientSecrets = clientSecrets
            });

            var codeReceiver = new LocalServerCodeReceiver();
            var authCode = new AuthorizationCodeInstalledApp(codeFlow, codeReceiver);

            var credential = await authCode.AuthorizeAsync(userName, CancellationToken.None);
            if (credential.Token.IsExpired(SystemClock.Default))
            {
                await credential.RefreshTokenAsync(CancellationToken.None);
            }

            var oauth2 = new SaslMechanismOAuth2(credential.UserId, credential.Token.AccessToken);

            this.client = new ImapClient();
            cancel = new CancellationTokenSource();
            await client.ConnectAsync("imap.gmail.com", 993, SecureSocketOptions.SslOnConnect, cancel.Token);
            await client.AuthenticateAsync(oauth2, cancel.Token);

            FetchMessages();
        }

        public void LogOut()
        {
            while (!client.IsConnected)
            {
                try
                {
                    cancel.Cancel();
                    client.Disconnect(true);
                }
                catch { }
            }
            cancel.Dispose();
        }
        public void FetchMessageContent(UniqueId messageId)
        {
            Task.Run(async () =>
            {
                try
                {
                    await _stateAsyncLock.WaitAsync(cancel.Token);
                    var result = (await folder.GetMessageAsync(messageId, cancel.Token)).HtmlBody;
                    App.Current.Dispatcher.BeginInvoke(new Action(() => FetchedMessageContent?.Invoke(this, result)), DispatcherPriority.Background);
                    _stateAsyncLock.Release();
                }
                catch(Exception ex)
                {
                    _stateAsyncLock.Release();
                }
            }, cancel.Token);
        }

        private void FetchMessages()
        {
            ObservableCollection<Message> msgs = new ObservableCollection<Message>();
            Messages = new C1DataCollection<Message>(msgs);
            Task.Run(new Action(async () =>
            {
                folder = client.GetFolder(MailKit.SpecialFolder.All);
                folder.Open(MailKit.FolderAccess.ReadOnly);
                TotalCount = Math.Min(1000, folder.Count);
                int chunkSize = 20;
                var uids = folder.Search(SearchQuery.All, cancel.Token).Reverse().Take(TotalCount).ToList();

                for (int chunk = 0; chunk <= TotalCount / chunkSize; chunk++)
                {
                    try
                    {
                        await _stateAsyncLock.WaitAsync(cancel.Token);
                        var messageSummaries = await folder.FetchAsync(uids.Skip(chunk * chunkSize).Take(chunkSize).ToList(), MailKit.MessageSummaryItems.All | MessageSummaryItems.PreviewText, cancel.Token);
                        _stateAsyncLock.Release();
                        messageSummaries.Reverse().ToList().ForEach(s =>
                        {
                            var msg = MapMessageSummaryToMessage(s as MessageSummary);
                            App.Current.Dispatcher.Invoke(new Action(() =>
                            {
                                msgs.Add(msg);
                                FetchedMessages?.Invoke(this, msgs.Count);
                            }));
                        });
                    }
                    catch{ }
                }
            }), cancel.Token);
        }        
        private Message MapMessageSummaryToMessage(MessageSummary messageSummary)
        {
            return new Message(messageSummary.UniqueId)
            {
                From = (messageSummary.Envelope.From.First() as MimeKit.MailboxAddress)?.Address,
                To = (messageSummary.Envelope.To.FirstOrDefault() as MimeKit.MailboxAddress)?.Address,
                Date = DateTime.FromFileTime(messageSummary.Date.ToFileTime()),
                Body = messageSummary.PreviewText,
                Subject = messageSummary.NormalizedSubject,
                SendByMe = (messageSummary.Envelope.From.First() as MimeKit.MailboxAddress)?.Address == User
            };
        }
        private void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
