﻿using FilterEditorDemo.Models;
using FilterEditorDemo.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Navigation;

namespace FilterEditorDemo
{
    public class LoginPageViewModel
    {
        private Regex _gmailRegex = new Regex(@"^[\w-\.]+@gmail.com$");
        public LoginModel Model { get; private set; }
        public ICommand LoginCommand { get; private set; }

        public event EventHandler<bool> SuccessfullLogIn;

        public LoginPageViewModel()
        {
            Model = new LoginModel();
            LoginCommand = new SimpleCommand()
            {
                ExecuteDelegate = async (x) => { await PerformLogin(x); },
                CanExecuteDelegate = (x) => { return AreDetailsValid(x); }
            };
        }

        private bool AreDetailsValid(object parameter)
        {
            bool result = true;
            result = result && Model.EmailId != null && _gmailRegex.IsMatch(Model.EmailId);
            if (!Model.UsingGmailAPI)
                result = result && !string.IsNullOrEmpty((parameter as PasswordBox).Password?.ToString());

            return result;
        }

        private async Task PerformLogin(object parameter)
        {
            try
            {
                if (Model.UsingGmailAPI)
                    await UserService.GetInstance().LoginWithGmail(Model.EmailId);
                else
                    await UserService.GetInstance().Login(Model.EmailId, (parameter as PasswordBox).Password.Trim());

                SuccessfullLogIn?.Invoke(this, true);
            }
            catch(Exception ex)
            {
                SuccessfullLogIn?.Invoke(this, false);
                MessageBox.Show(!Model.UsingGmailAPI? @"
In case credentials are correct: 
 - Google did not allow to login for security reasons.
 - Please login securely by using the provided Gmail API option." : "Please provide correct details", ex.Message);
            }
        }
    }
    public class SimpleCommand : ICommand
    {
        public Predicate<object> CanExecuteDelegate { get; set; }
        public Action<object> ExecuteDelegate { get; set; }

        #region ICommand Members

        public bool CanExecute(object parameter)
        {
            if (CanExecuteDelegate != null)
                return CanExecuteDelegate(parameter);
            return true;
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            if (ExecuteDelegate != null)
                ExecuteDelegate(parameter);
        }

        #endregion
    }
}
