﻿using C1.DataFilter;
using C1.WPF.DataFilter;
using C1.WPF.Extended;
using C1.WPF.RichTextBox;
using FilterEditorDemo.CustomControls;
using FilterEditorDemo.Models;
using FilterEditorDemo.Service;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace FilterEditorDemo
{
    /// <summary>
    /// Interaction logic for InboxPage.xaml
    /// </summary>
    public partial class InboxPage : Page
    {
        UserService UserService { get; }
        public InboxPage()
        {
            UserService = UserService.GetInstance();
            UserService.FetchedMessages += UserService_FetchedMessages;
            UserService.FetchedMessageContent += UserService_FetchedMessageContent;
            InitializeComponent();

            popup.PlacementTarget = this;
            this.DataContext = UserService;
            InitFilters();
        }
        private void InitFilters()
        {
            dataFilter.Filters.Add(new DateRangeFilter("Date")
            {
                HeaderText = "Date Range",
                Minimum = DateTime.Now.AddDays(-500),
                Maximum = DateTime.Now.AddDays(1),
            });

            dataFilter.Filters.Add(new RuleBasedFilter(UserService.Messages));
        }

        private void Logout_Click(object sender, RoutedEventArgs e)
        {
            UserService.LogOut();
            NavigationService.Navigate(new Uri("Views/LoginPage.xaml", UriKind.Relative));
        }

        private void ListBox_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (msgsList.SelectedItem != null)
            {
                DisplayPopup(msgsList.SelectedItem as Message);
            }
        }

        private void MessageControl_DisplayPopup(object sender, object e)
        {
            DisplayPopup(e as Message);
        }
        private void DisplayPopup(Message message)
        {
            spinner.IsActive = true;
            UserService.FetchMessageContent(message.GetId());
        }
        private void UserService_FetchedMessageContent(object sender, string e)
        {
            spinner.IsActive = false;
            var rtb = popupCtrl.Child as C1RichTextBox;
            try
            {
                rtb.HorizontalOffset = rtb.VerticalOffset = 0;
                rtb.Html = e;
                popup.IsOpen = true;
            }
            catch (Exception)
            {
                popup.IsOpen = false;
                rtb.Text = "Content can't be displayed.";
                popup.IsOpen = true;
            }
        }
        private void UserService_FetchedMessages(object sender, int e)
        {
            progress.Value = e;
            if (progress.Value == progress.Maximum)
                progress.Visibility = Visibility.Collapsed;
        }
        private void PopupCtrl_ClosePopup(object sender, EventArgs e)
        {
            popup.IsOpen = false;
        }
    }
    public class VirtualizingItemPresenter : ItemsPresenter
    {
        protected override void OnTemplateChanged(ItemsPanelTemplate oldTemplate, ItemsPanelTemplate newTemplate)
        {
            base.OnTemplateChanged(oldTemplate, new ItemsPanelTemplate(new FrameworkElementFactory(typeof(VirtualizingStackPanel))));
        }
    }

    #region Rule based filter
    public class RuleBasedFilter : CustomFilter
    {
        RuleTree _ruleTree;
        public RuleBasedFilter(IEnumerable source) : base()
        {
            Control = _ruleTree = new RuleTree() { Source = source};

            _ruleTree.RuleChanged += (s, e) => 
            {
                OnValueChanged(new ValueChangedEventArgs() { ApplyFilter = true });
            };
        }
        protected override C1.DataFilter.Expression GetExpression()
        {
            return _ruleTree.Expression;
        }

        protected override void SetExpression(C1.DataFilter.Expression expression)
        {
            
        }
        public override bool IsApplied => _ruleTree.Expression!=null;
    }
    #endregion
}
